use super::*;
use std::any::Any;

#[derive(Debug)]
pub enum CommandKind
{
    /// Shutdown gracefully.
    ///
    /// # Response
    /// None.
    GracefulShutdown,
}

/// A response from the interrupt channel.
/// Some command kinds may warrant a response.
pub type CommandResponse = Box<dyn Any + Send + 'static>;

/// A command to interrupt the web background task.
#[derive(Debug)]
pub(super) struct Command
{
    pub(super) kind: CommandKind,
    pub(super) response: oneshot::Sender<CommandResponse>, // If the interrupt stream produces no response for this query, the sender will just be dropped and the receiver will `Err`.
}

/// A channel to communicate with background task
#[derive(Debug, Clone)]
pub struct InterruptChannel(pub(super) mpsc::Sender<Command>);
